# This file was automatically generated by SWIG (http://www.swig.org).
# Version 1.3.40
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.
# This file is compatible with both classic and new-style classes.

from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_clinkgrammar', [dirname(__file__)])
        except ImportError:
            import _clinkgrammar
            return _clinkgrammar
        if fp is not None:
            try:
                _mod = imp.load_module('_clinkgrammar', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _clinkgrammar = swig_import_helper()
    del swig_import_helper
else:
    import _clinkgrammar
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static) or hasattr(self,name):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0



def linkgrammar_get_version():
  return _clinkgrammar.linkgrammar_get_version()
linkgrammar_get_version = _clinkgrammar.linkgrammar_get_version

def dictionary_create(*args):
  return _clinkgrammar.dictionary_create(*args)
dictionary_create = _clinkgrammar.dictionary_create

def dictionary_create_lang(*args):
  return _clinkgrammar.dictionary_create_lang(*args)
dictionary_create_lang = _clinkgrammar.dictionary_create_lang

def dictionary_create_default_lang():
  return _clinkgrammar.dictionary_create_default_lang()
dictionary_create_default_lang = _clinkgrammar.dictionary_create_default_lang

def dictionary_delete(*args):
  return _clinkgrammar.dictionary_delete(*args)
dictionary_delete = _clinkgrammar.dictionary_delete

def dictionary_get_max_cost(*args):
  return _clinkgrammar.dictionary_get_max_cost(*args)
dictionary_get_max_cost = _clinkgrammar.dictionary_get_max_cost

def dictionary_set_data_dir(*args):
  return _clinkgrammar.dictionary_set_data_dir(*args)
dictionary_set_data_dir = _clinkgrammar.dictionary_set_data_dir

def dictionary_get_data_dir():
  return _clinkgrammar.dictionary_get_data_dir()
dictionary_get_data_dir = _clinkgrammar.dictionary_get_data_dir

def dictionary_is_past_tense_form(*args):
  return _clinkgrammar.dictionary_is_past_tense_form(*args)
dictionary_is_past_tense_form = _clinkgrammar.dictionary_is_past_tense_form

def dictionary_is_entity(*args):
  return _clinkgrammar.dictionary_is_entity(*args)
dictionary_is_entity = _clinkgrammar.dictionary_is_entity

def parse_options_create():
  return _clinkgrammar.parse_options_create()
parse_options_create = _clinkgrammar.parse_options_create

def parse_options_delete(*args):
  return _clinkgrammar.parse_options_delete(*args)
parse_options_delete = _clinkgrammar.parse_options_delete

def parse_options_set_verbosity(*args):
  return _clinkgrammar.parse_options_set_verbosity(*args)
parse_options_set_verbosity = _clinkgrammar.parse_options_set_verbosity

def parse_options_get_verbosity(*args):
  return _clinkgrammar.parse_options_get_verbosity(*args)
parse_options_get_verbosity = _clinkgrammar.parse_options_get_verbosity

def parse_options_set_linkage_limit(*args):
  return _clinkgrammar.parse_options_set_linkage_limit(*args)
parse_options_set_linkage_limit = _clinkgrammar.parse_options_set_linkage_limit

def parse_options_get_linkage_limit(*args):
  return _clinkgrammar.parse_options_get_linkage_limit(*args)
parse_options_get_linkage_limit = _clinkgrammar.parse_options_get_linkage_limit

def parse_options_set_disjunct_cost(*args):
  return _clinkgrammar.parse_options_set_disjunct_cost(*args)
parse_options_set_disjunct_cost = _clinkgrammar.parse_options_set_disjunct_cost

def parse_options_get_disjunct_cost(*args):
  return _clinkgrammar.parse_options_get_disjunct_cost(*args)
parse_options_get_disjunct_cost = _clinkgrammar.parse_options_get_disjunct_cost

def parse_options_set_min_null_count(*args):
  return _clinkgrammar.parse_options_set_min_null_count(*args)
parse_options_set_min_null_count = _clinkgrammar.parse_options_set_min_null_count

def parse_options_get_min_null_count(*args):
  return _clinkgrammar.parse_options_get_min_null_count(*args)
parse_options_get_min_null_count = _clinkgrammar.parse_options_get_min_null_count

def parse_options_set_max_null_count(*args):
  return _clinkgrammar.parse_options_set_max_null_count(*args)
parse_options_set_max_null_count = _clinkgrammar.parse_options_set_max_null_count

def parse_options_get_max_null_count(*args):
  return _clinkgrammar.parse_options_get_max_null_count(*args)
parse_options_get_max_null_count = _clinkgrammar.parse_options_get_max_null_count

def parse_options_set_null_block(*args):
  return _clinkgrammar.parse_options_set_null_block(*args)
parse_options_set_null_block = _clinkgrammar.parse_options_set_null_block

def parse_options_get_null_block(*args):
  return _clinkgrammar.parse_options_get_null_block(*args)
parse_options_get_null_block = _clinkgrammar.parse_options_get_null_block

def parse_options_set_islands_ok(*args):
  return _clinkgrammar.parse_options_set_islands_ok(*args)
parse_options_set_islands_ok = _clinkgrammar.parse_options_set_islands_ok

def parse_options_get_islands_ok(*args):
  return _clinkgrammar.parse_options_get_islands_ok(*args)
parse_options_get_islands_ok = _clinkgrammar.parse_options_get_islands_ok

def parse_options_set_short_length(*args):
  return _clinkgrammar.parse_options_set_short_length(*args)
parse_options_set_short_length = _clinkgrammar.parse_options_set_short_length

def parse_options_get_short_length(*args):
  return _clinkgrammar.parse_options_get_short_length(*args)
parse_options_get_short_length = _clinkgrammar.parse_options_get_short_length

def parse_options_set_max_memory(*args):
  return _clinkgrammar.parse_options_set_max_memory(*args)
parse_options_set_max_memory = _clinkgrammar.parse_options_set_max_memory

def parse_options_get_max_memory(*args):
  return _clinkgrammar.parse_options_get_max_memory(*args)
parse_options_get_max_memory = _clinkgrammar.parse_options_get_max_memory

def parse_options_set_max_sentence_length(*args):
  return _clinkgrammar.parse_options_set_max_sentence_length(*args)
parse_options_set_max_sentence_length = _clinkgrammar.parse_options_set_max_sentence_length

def parse_options_get_max_sentence_length(*args):
  return _clinkgrammar.parse_options_get_max_sentence_length(*args)
parse_options_get_max_sentence_length = _clinkgrammar.parse_options_get_max_sentence_length

def parse_options_set_max_parse_time(*args):
  return _clinkgrammar.parse_options_set_max_parse_time(*args)
parse_options_set_max_parse_time = _clinkgrammar.parse_options_set_max_parse_time

def parse_options_get_max_parse_time(*args):
  return _clinkgrammar.parse_options_get_max_parse_time(*args)
parse_options_get_max_parse_time = _clinkgrammar.parse_options_get_max_parse_time

def parse_options_set_cost_model_type(*args):
  return _clinkgrammar.parse_options_set_cost_model_type(*args)
parse_options_set_cost_model_type = _clinkgrammar.parse_options_set_cost_model_type

def parse_options_get_cost_model_type(*args):
  return _clinkgrammar.parse_options_get_cost_model_type(*args)
parse_options_get_cost_model_type = _clinkgrammar.parse_options_get_cost_model_type

def parse_options_timer_expired(*args):
  return _clinkgrammar.parse_options_timer_expired(*args)
parse_options_timer_expired = _clinkgrammar.parse_options_timer_expired

def parse_options_memory_exhausted(*args):
  return _clinkgrammar.parse_options_memory_exhausted(*args)
parse_options_memory_exhausted = _clinkgrammar.parse_options_memory_exhausted

def parse_options_resources_exhausted(*args):
  return _clinkgrammar.parse_options_resources_exhausted(*args)
parse_options_resources_exhausted = _clinkgrammar.parse_options_resources_exhausted

def parse_options_set_screen_width(*args):
  return _clinkgrammar.parse_options_set_screen_width(*args)
parse_options_set_screen_width = _clinkgrammar.parse_options_set_screen_width

def parse_options_get_screen_width(*args):
  return _clinkgrammar.parse_options_get_screen_width(*args)
parse_options_get_screen_width = _clinkgrammar.parse_options_get_screen_width

def parse_options_set_allow_null(*args):
  return _clinkgrammar.parse_options_set_allow_null(*args)
parse_options_set_allow_null = _clinkgrammar.parse_options_set_allow_null

def parse_options_get_allow_null(*args):
  return _clinkgrammar.parse_options_get_allow_null(*args)
parse_options_get_allow_null = _clinkgrammar.parse_options_get_allow_null

def parse_options_set_display_walls(*args):
  return _clinkgrammar.parse_options_set_display_walls(*args)
parse_options_set_display_walls = _clinkgrammar.parse_options_set_display_walls

def parse_options_get_display_walls(*args):
  return _clinkgrammar.parse_options_get_display_walls(*args)
parse_options_get_display_walls = _clinkgrammar.parse_options_get_display_walls

def parse_options_set_all_short_connectors(*args):
  return _clinkgrammar.parse_options_set_all_short_connectors(*args)
parse_options_set_all_short_connectors = _clinkgrammar.parse_options_set_all_short_connectors

def parse_options_get_all_short_connectors(*args):
  return _clinkgrammar.parse_options_get_all_short_connectors(*args)
parse_options_get_all_short_connectors = _clinkgrammar.parse_options_get_all_short_connectors

def parse_options_reset_resources(*args):
  return _clinkgrammar.parse_options_reset_resources(*args)
parse_options_reset_resources = _clinkgrammar.parse_options_reset_resources

def sentence_create(*args):
  return _clinkgrammar.sentence_create(*args)
sentence_create = _clinkgrammar.sentence_create

def sentence_delete(*args):
  return _clinkgrammar.sentence_delete(*args)
sentence_delete = _clinkgrammar.sentence_delete

def sentence_parse(*args):
  return _clinkgrammar.sentence_parse(*args)
sentence_parse = _clinkgrammar.sentence_parse

def sentence_length(*args):
  return _clinkgrammar.sentence_length(*args)
sentence_length = _clinkgrammar.sentence_length

def sentence_get_word(*args):
  return _clinkgrammar.sentence_get_word(*args)
sentence_get_word = _clinkgrammar.sentence_get_word

def sentence_null_count(*args):
  return _clinkgrammar.sentence_null_count(*args)
sentence_null_count = _clinkgrammar.sentence_null_count

def sentence_num_linkages_found(*args):
  return _clinkgrammar.sentence_num_linkages_found(*args)
sentence_num_linkages_found = _clinkgrammar.sentence_num_linkages_found

def sentence_num_valid_linkages(*args):
  return _clinkgrammar.sentence_num_valid_linkages(*args)
sentence_num_valid_linkages = _clinkgrammar.sentence_num_valid_linkages

def sentence_num_linkages_post_processed(*args):
  return _clinkgrammar.sentence_num_linkages_post_processed(*args)
sentence_num_linkages_post_processed = _clinkgrammar.sentence_num_linkages_post_processed

def sentence_num_violations(*args):
  return _clinkgrammar.sentence_num_violations(*args)
sentence_num_violations = _clinkgrammar.sentence_num_violations

def sentence_and_cost(*args):
  return _clinkgrammar.sentence_and_cost(*args)
sentence_and_cost = _clinkgrammar.sentence_and_cost

def sentence_disjunct_cost(*args):
  return _clinkgrammar.sentence_disjunct_cost(*args)
sentence_disjunct_cost = _clinkgrammar.sentence_disjunct_cost

def sentence_link_cost(*args):
  return _clinkgrammar.sentence_link_cost(*args)
sentence_link_cost = _clinkgrammar.sentence_link_cost

def sentence_get_nth_word(*args):
  return _clinkgrammar.sentence_get_nth_word(*args)
sentence_get_nth_word = _clinkgrammar.sentence_get_nth_word

def sentence_nth_word_has_disjunction(*args):
  return _clinkgrammar.sentence_nth_word_has_disjunction(*args)
sentence_nth_word_has_disjunction = _clinkgrammar.sentence_nth_word_has_disjunction

def linkage_create(*args):
  return _clinkgrammar.linkage_create(*args)
linkage_create = _clinkgrammar.linkage_create

def linkage_delete(*args):
  return _clinkgrammar.linkage_delete(*args)
linkage_delete = _clinkgrammar.linkage_delete

def linkage_print_diagram(*args):
  return _clinkgrammar.linkage_print_diagram(*args)
linkage_print_diagram = _clinkgrammar.linkage_print_diagram

def linkage_get_current_sublinkage(*args):
  return _clinkgrammar.linkage_get_current_sublinkage(*args)
linkage_get_current_sublinkage = _clinkgrammar.linkage_get_current_sublinkage

def linkage_set_current_sublinkage(*args):
  return _clinkgrammar.linkage_set_current_sublinkage(*args)
linkage_set_current_sublinkage = _clinkgrammar.linkage_set_current_sublinkage

def linkage_get_sentence(*args):
  return _clinkgrammar.linkage_get_sentence(*args)
linkage_get_sentence = _clinkgrammar.linkage_get_sentence

def linkage_get_num_sublinkages(*args):
  return _clinkgrammar.linkage_get_num_sublinkages(*args)
linkage_get_num_sublinkages = _clinkgrammar.linkage_get_num_sublinkages

def linkage_get_num_words(*args):
  return _clinkgrammar.linkage_get_num_words(*args)
linkage_get_num_words = _clinkgrammar.linkage_get_num_words

def linkage_get_num_links(*args):
  return _clinkgrammar.linkage_get_num_links(*args)
linkage_get_num_links = _clinkgrammar.linkage_get_num_links

def linkage_get_link_lword(*args):
  return _clinkgrammar.linkage_get_link_lword(*args)
linkage_get_link_lword = _clinkgrammar.linkage_get_link_lword

def linkage_get_link_rword(*args):
  return _clinkgrammar.linkage_get_link_rword(*args)
linkage_get_link_rword = _clinkgrammar.linkage_get_link_rword

def linkage_get_link_length(*args):
  return _clinkgrammar.linkage_get_link_length(*args)
linkage_get_link_length = _clinkgrammar.linkage_get_link_length

def linkage_get_link_label(*args):
  return _clinkgrammar.linkage_get_link_label(*args)
linkage_get_link_label = _clinkgrammar.linkage_get_link_label

def linkage_get_link_llabel(*args):
  return _clinkgrammar.linkage_get_link_llabel(*args)
linkage_get_link_llabel = _clinkgrammar.linkage_get_link_llabel

def linkage_get_link_rlabel(*args):
  return _clinkgrammar.linkage_get_link_rlabel(*args)
linkage_get_link_rlabel = _clinkgrammar.linkage_get_link_rlabel

def linkage_get_link_num_domains(*args):
  return _clinkgrammar.linkage_get_link_num_domains(*args)
linkage_get_link_num_domains = _clinkgrammar.linkage_get_link_num_domains

def linkage_get_link_domain_names(*args):
  return _clinkgrammar.linkage_get_link_domain_names(*args)
linkage_get_link_domain_names = _clinkgrammar.linkage_get_link_domain_names

def linkage_get_words(*args):
  return _clinkgrammar.linkage_get_words(*args)
linkage_get_words = _clinkgrammar.linkage_get_words

def linkage_get_word(*args):
  return _clinkgrammar.linkage_get_word(*args)
linkage_get_word = _clinkgrammar.linkage_get_word

def linkage_print_links_and_domains(*args):
  return _clinkgrammar.linkage_print_links_and_domains(*args)
linkage_print_links_and_domains = _clinkgrammar.linkage_print_links_and_domains

def linkage_print_senses(*args):
  return _clinkgrammar.linkage_print_senses(*args)
linkage_print_senses = _clinkgrammar.linkage_print_senses

def linkage_print_constituent_tree(*args):
  return _clinkgrammar.linkage_print_constituent_tree(*args)
linkage_print_constituent_tree = _clinkgrammar.linkage_print_constituent_tree

def linkage_print_postscript(*args):
  return _clinkgrammar.linkage_print_postscript(*args)
linkage_print_postscript = _clinkgrammar.linkage_print_postscript

def linkage_compute_union(*args):
  return _clinkgrammar.linkage_compute_union(*args)
linkage_compute_union = _clinkgrammar.linkage_compute_union

def linkage_unused_word_cost(*args):
  return _clinkgrammar.linkage_unused_word_cost(*args)
linkage_unused_word_cost = _clinkgrammar.linkage_unused_word_cost

def linkage_disjunct_cost(*args):
  return _clinkgrammar.linkage_disjunct_cost(*args)
linkage_disjunct_cost = _clinkgrammar.linkage_disjunct_cost

def linkage_and_cost(*args):
  return _clinkgrammar.linkage_and_cost(*args)
linkage_and_cost = _clinkgrammar.linkage_and_cost

def linkage_link_cost(*args):
  return _clinkgrammar.linkage_link_cost(*args)
linkage_link_cost = _clinkgrammar.linkage_link_cost

def linkage_corpus_cost(*args):
  return _clinkgrammar.linkage_corpus_cost(*args)
linkage_corpus_cost = _clinkgrammar.linkage_corpus_cost

def linkage_is_canonical(*args):
  return _clinkgrammar.linkage_is_canonical(*args)
linkage_is_canonical = _clinkgrammar.linkage_is_canonical

def linkage_is_improper(*args):
  return _clinkgrammar.linkage_is_improper(*args)
linkage_is_improper = _clinkgrammar.linkage_is_improper

def linkage_has_inconsistent_domains(*args):
  return _clinkgrammar.linkage_has_inconsistent_domains(*args)
linkage_has_inconsistent_domains = _clinkgrammar.linkage_has_inconsistent_domains

def linkage_get_violation_name(*args):
  return _clinkgrammar.linkage_get_violation_name(*args)
linkage_get_violation_name = _clinkgrammar.linkage_get_violation_name

def post_process_open(*args):
  return _clinkgrammar.post_process_open(*args)
post_process_open = _clinkgrammar.post_process_open

def post_process_close(*args):
  return _clinkgrammar.post_process_close(*args)
post_process_close = _clinkgrammar.post_process_close

def linkage_post_process(*args):
  return _clinkgrammar.linkage_post_process(*args)
linkage_post_process = _clinkgrammar.linkage_post_process

def linkage_constituent_tree(*args):
  return _clinkgrammar.linkage_constituent_tree(*args)
linkage_constituent_tree = _clinkgrammar.linkage_constituent_tree

def linkage_free_constituent_tree(*args):
  return _clinkgrammar.linkage_free_constituent_tree(*args)
linkage_free_constituent_tree = _clinkgrammar.linkage_free_constituent_tree

def linkage_constituent_node_get_label(*args):
  return _clinkgrammar.linkage_constituent_node_get_label(*args)
linkage_constituent_node_get_label = _clinkgrammar.linkage_constituent_node_get_label

def linkage_constituent_node_get_child(*args):
  return _clinkgrammar.linkage_constituent_node_get_child(*args)
linkage_constituent_node_get_child = _clinkgrammar.linkage_constituent_node_get_child

def linkage_constituent_node_get_next(*args):
  return _clinkgrammar.linkage_constituent_node_get_next(*args)
linkage_constituent_node_get_next = _clinkgrammar.linkage_constituent_node_get_next

def linkage_constituent_node_get_start(*args):
  return _clinkgrammar.linkage_constituent_node_get_start(*args)
linkage_constituent_node_get_start = _clinkgrammar.linkage_constituent_node_get_start

def linkage_constituent_node_get_end(*args):
  return _clinkgrammar.linkage_constituent_node_get_end(*args)
linkage_constituent_node_get_end = _clinkgrammar.linkage_constituent_node_get_end





from itertools import chain
import unittest

from linkgrammar import Parser, Linkage, ConstituentNode, ParseOptions, Link
import clinkgrammar as clg

class ParseOptionsTestCase(unittest.TestCase):
    def test_setting_verbosity(self):
        po = ParseOptions()
        po.verbosity = 2
        #Ensure that the PO object reports the value correctly
        self.assertEqual(po.verbosity, 2)
        #Ensure that it's actually setting it.
        self.assertEqual(clg.parse_options_get_verbosity(po._po), 2)
        
    def test_setting_verbosity_to_not_allow_value_raises_value_error(self):
        po = ParseOptions()
        self.assertRaises(ValueError, setattr, po, "verbosity", 4)
        
    def test_setting_verbosity_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "verbosity", "a")

    def test_setting_linkage_limit(self):
        po = ParseOptions()
        po.linkage_limit = 3
        self.assertEqual(clg.parse_options_get_linkage_limit(po._po), 3)
        
    def test_setting_linkage_limit_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "linkage_limit", "a")

    def test_setting_linkage_limit_to_negative_number_raises_value_error(self):
        po = ParseOptions()
        self.assertRaises(ValueError, setattr, po, "linkage_limit", -1)
        
    def test_setting_disjunct_cost(self):
        po = ParseOptions()
        po.disjunct_cost = 3
        self.assertEqual(clg.parse_options_get_disjunct_cost(po._po), 3)
        
    def test_setting_disjunct_cost_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "disjunct_cost", "a")

    def test_setting_min_null_count(self):
        po = ParseOptions()
        po.min_null_count = 3
        self.assertEqual(clg.parse_options_get_min_null_count(po._po), 3)
        
    def test_setting_min_null_count_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "min_null_count", "a")

    def test_setting_min_null_count_to_negative_number_raises_value_error(self):
        po = ParseOptions()
        self.assertRaises(ValueError, setattr, po, "min_null_count", -1)

    def test_setting_max_null_count(self):
        po = ParseOptions()
        po.max_null_count = 3
        self.assertEqual(clg.parse_options_get_max_null_count(po._po), 3)
        
    def test_setting_max_null_count_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "max_null_count", "a")

    def test_setting_max_null_count_to_negative_number_raises_value_error(self):
        po = ParseOptions()
        self.assertRaises(ValueError, setattr, po, "max_null_count", -1)
        
    def test_setting_null_block(self):
        po = ParseOptions()
        po.null_block = 3
        self.assertEqual(clg.parse_options_get_null_block(po._po), 3)
        
    def test_setting_null_block_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "null_block", "a")

    def test_setting_null_block_to_negative_number_raises_value_error(self):
        po = ParseOptions()
        self.assertRaises(ValueError, setattr, po, "null_block", -1)
        
    def test_setting_short_length(self):
        po = ParseOptions()
        po.short_length = 3
        self.assertEqual(clg.parse_options_get_short_length(po._po), 3)
        
    def test_setting_short_length_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "short_length", "a")

    def test_setting_short_length_to_negative_number_raises_value_error(self):
        po = ParseOptions()
        self.assertRaises(ValueError, setattr, po, "short_length", -1)
        
    def test_setting_max_sentence_length(self):
        po = ParseOptions()
        po.max_sentence_length = 3
        self.assertEqual(clg.parse_options_get_max_sentence_length(po._po), 3)
        
    def test_setting_max_sentence_length_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "max_sentence_length", "a")

    def test_setting_max_sentence_length_to_negative_number_raises_value_error(self):
        po = ParseOptions()
        self.assertRaises(ValueError, setattr, po, "max_sentence_length", -1)
        
    def test_setting_islands_ok(self):
        po = ParseOptions()
        po.islands_ok = True
        self.assertEqual(po.islands_ok, True)
        self.assertEqual(clg.parse_options_get_islands_ok(po._po), 1)
        po.islands_ok = False
        self.assertEqual(po.islands_ok, False)
        self.assertEqual(clg.parse_options_get_islands_ok(po._po), 0)
    
    def test_setting_islands_ok_to_non_boolean_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "islands_ok", "a")
        
    def test_setting_max_memory(self):
        po = ParseOptions()
        po.max_memory = 3
        self.assertEqual(clg.parse_options_get_max_memory(po._po), 3)
        
    def test_setting_max_memory_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "max_memory", "a")

    def test_setting_max_memory_to_negative_number_raises_value_error(self):
        po = ParseOptions()
        self.assertRaises(ValueError, setattr, po, "max_memory", -1)
        
    def test_setting_max_parse_time(self):
        po = ParseOptions()
        po.max_parse_time = 3
        self.assertEqual(clg.parse_options_get_max_parse_time(po._po), 3)
        
    def test_setting_max_parse_time_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "max_parse_time", "a")
        
    def test_setting_screen_width(self):
        po = ParseOptions()
        po.screen_width = 3
        self.assertEqual(clg.parse_options_get_screen_width(po._po), 3)
        
    def test_setting_screen_width_to_non_integer_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "screen_width", "a")

    def test_setting_screen_width_to_negative_number_raises_value_error(self):
        po = ParseOptions()
        self.assertRaises(ValueError, setattr, po, "screen_width", -1)
        
    def test_setting_allow_null(self):
        po = ParseOptions()
        po.allow_null = True
        self.assertEqual(po.allow_null, True)
        self.assertEqual(clg.parse_options_get_allow_null(po._po), 1)
        po.allow_null = False
        self.assertEqual(po.allow_null, False)
        self.assertEqual(clg.parse_options_get_allow_null(po._po), 0)
    
    def test_setting_allow_null_to_non_boolean_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "allow_null", "a")
        
    def test_setting_display_walls(self):
        po = ParseOptions()
        po.display_walls = True
        self.assertEqual(po.display_walls, True)
        self.assertEqual(clg.parse_options_get_display_walls(po._po), 1)
        po.display_walls = False
        self.assertEqual(po.display_walls, False)
        self.assertEqual(clg.parse_options_get_display_walls(po._po), 0)
    
    def test_setting_display_walls_to_non_boolean_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "display_walls", "a")
        
    def test_setting_all_short_connectors(self):
        po = ParseOptions()
        po.all_short_connectors = True
        self.assertEqual(po.all_short_connectors, True)
        self.assertEqual(clg.parse_options_get_all_short_connectors(po._po), 1)
        po.all_short_connectors = False
        self.assertEqual(po.all_short_connectors, False)
        self.assertEqual(clg.parse_options_get_all_short_connectors(po._po), 0)
    
    def test_setting_all_short_connectors_to_non_boolean_raises_type_error(self):
        po = ParseOptions()
        self.assertRaises(TypeError, setattr, po, "all_short_connectors", "a")
        
            
class ParserTestCase(unittest.TestCase):
    def test_specifying_options_when_instantiating_parser(self):
        p = Parser(linkage_limit=10)
        self.assertEqual(clg.parse_options_get_linkage_limit(p.parse_options._po), 10)
        
    def test_that_parser_can_be_destroyed_when_linkages_still_exist(self):
        """
        Unless the swig objects associated with sentences, linkages, etc. are free'd before
        the parser is deleted linkrgammar will crap its pants.  This test ensures that
        parsers can be created and deleted without regard for the existence of PYTHON Linkage
        objects
        """
        p = Parser()
        linkages = p.parse_sent('This is a sentence.')
        del p
        
        
        
class ParsingTestCase(unittest.TestCase):
    def setUp(self):
        self.p = Parser()
        
    def test_that_parse_sent_returns_list_of_linkage_objects_for_valid_sentence(self):
        result = self.p.parse_sent("This is a sentence.")
        self.assertTrue(isinstance(result, list))
        self.assertTrue(isinstance(result[0], Linkage))
        self.assertTrue(isinstance(result[1], Linkage))
        
    def test_utf8_encoded_string(self):
        result = self.p.parse_sent(u"I love going to the caf\N{LATIN SMALL LETTER E WITH ACUTE}.".encode('utf8'))
        self.assertTrue(isinstance(result, list))
        self.assertTrue(isinstance(result[0], Linkage))
        self.assertTrue(isinstance(result[1], Linkage))
        
    def test_getting_link_distances(self):
        result = self.p.parse_sent("This is a sentence.")[0]
        self.assertEqual(result.link_distances, [5,1,1,2,1,1])
        result = self.p.parse_sent("This is a silly sentence.")[0]
        self.assertEqual(result.link_distances, [6,1,1,3,2,1,1])

class LinkageTestCase(unittest.TestCase):
    def setUp(self):
        self.p = Parser()

    def test_getting_words(self):
        self.assertEqual(self.p.parse_sent('This is a sentence.')[0].words,
                         ['LEFT-WALL', 'this.p', 'is.v', 'a', 'sentence.n', '.', 'RIGHT-WALL'])
        
    def test_getting_num_of_words(self):
        #Words include punctuation and a 'LEFT-WALL' and 'RIGHT_WALL'
        self.assertEqual(self.p.parse_sent('This is a sentence.')[0].num_of_words, 7)
        
    def test_getting_links(self):
        sent = 'This is a sentence.'
        linkage = self.p.parse_sent(sent)[0]
        self.assertEqual(linkage.links[0],
                         Link('LEFT-WALL','Xp','Xp','.'))
        self.assertEqual(linkage.links[1],
                         Link('LEFT-WALL','Wd','Wd','this.p'))
        self.assertEqual(linkage.links[2],
                         Link('this.p','Ss*b','Ss','is.v'))
        self.assertEqual(linkage.links[3],
                         Link('is.v','O*t','Os','sentence.n'))
        self.assertEqual(linkage.links[4],
                         Link('a','Ds','Ds','sentence.n'))
        self.assertEqual(linkage.links[5],
                         Link('.','RW','RW','RIGHT-WALL'))
            
class SublinkageTestCase(unittest.TestCase):
    def setUp(self):
        self.p = Parser()
        
    def test_num_of_sublinkages(self):
        #REVIEW: By convention, linkgrammar says a sentence has one sublinkage if it has none.
        #Should we change it?
        sent = "This sentence has no sublinkages."
        self.assertEqual(self.p.parse_sent(sent)[0].num_of_sublinkages, 1)
        
        sent = "This sentence has one sublinkage and it is because of its coordinating conjunction."
        self.assertEqual(self.p.parse_sent(sent)[0].num_of_sublinkages, 2)
        
    def test_union_of_single_linkage_equal_to_itself(self):
        """
        For a sentence with no coordinating conjunctions, the union should be the same as the linkage itself
        """
        sent = "This sentence has no sublinkages."
        linkage = self.p.parse_sent(sent)[0]
        self.assertEqual(linkage.union.links, linkage.links)
        
    def test_union_of_linkage_with_sub_linkages_not_equal_to_itself(self):
        sent = "This sentence has one sublinkage and it is because of its coordinating conjunction."
        linkage = self.p.parse_sent(sent)[0]
        self.assertNotEqual(linkage.union.links, linkage.links)        
    
    def test_simple_sublinkages(self):
        """
        When there is a simple coordinating conjunction like 'and' used in this way,
        it's easy to see how sublinkages are constructed.  The rest of the sentence
        remains the same, but each sublinkage contains one of the clauses.
        """
        sent = "There are birds and bees."
        linkage = self.p.parse_sent(sent)[0]
        self.assertEqual(linkage.sublinkages[0].links,[
                         Link('LEFT-WALL','Xp','Xp','.'),
                         Link('LEFT-WALL','Wd','Wd','there'),
                         Link('there','SFp','SFp','are.v'),
                         Link('are.v','O*t','Op','birds.n'),
                         Link('.','RW','RW','RIGHT-WALL')
                         ])
        self.assertEqual(linkage.sublinkages[1].links,[
                         Link('LEFT-WALL','Xp','Xp','.'),
                         Link('LEFT-WALL','Wd','Wd','there'),
                         Link('there','SFp','SFp','are.v'),
                         Link('are.v','O*t','Op','bees.n'),
                         Link('.','RW','RW','RIGHT-WALL')
                         ])
    
class LinkTestCase(unittest.TestCase):
    def test_link_display_with_identical_link_type(self):
        self.assertEqual(unicode(Link('Left','Link','Link','Right')),
                         u'Left-Link-Right')

    def test_link_display_with_identical_link_type(self):
        self.assertEqual(unicode(Link('Left','Link','Link*','Right')),
                         u'Left-Link-Link*-Right')
        
class ConstituentPhraseTestCase(unittest.TestCase):
    def setUp(self):
        self.p = Parser()

    def test_simple_one_word_one_phrase_sentence(self):
        linkages = self.p.parse_sent("Go.")
        self.assertEqual(linkages[0].constituent_phrases_nested,
                         [ConstituentNode('S', words=['.']), [ConstituentNode('VP', words=['Go'])]])

    def test_simple_one_word_one_phrase_sentence_with_flattened_result(self):
        linkages = self.p.parse_sent("Go.")
        self.assertEqual(linkages[0].constituent_phrases_flat,
                         [ConstituentNode('S', words=['.']), ConstituentNode('VP', words=['Go'])])

unittest.main()


import locale

import clinkgrammar as clg

class Parser(object):
    def __init__(self, verbosity=0,
                       linkage_limit=100,
                       min_null_count=0,
                       max_null_count=0,
                       null_block=1,
                       islands_ok=False,
                       short_length=6,
                       all_short_connectors=False,
                       display_walls=False,
                       allow_null=True,
                       screen_width=180,
                       max_parse_time=5,
                       max_memory=128000000,
                       disjunct_cost=10000,
                       max_sentence_length=170):
        locale.setlocale(locale.LC_ALL,"en_US.UTF-8")
        
        #We need to retain all these options cause we will
        #recreate the parse_options if the parser starts failing
        self.verbosity = verbosity
        self.linkage_limit = linkage_limit
        self.min_null_count = min_null_count
        self.max_null_count = max_null_count
        self.null_block = null_block
        self.islands_ok = islands_ok
        self.short_length = short_length
        self.all_short_connectors = all_short_connectors
        self.display_walls = display_walls
        self.screen_width = screen_width
        self.max_parse_time = max_parse_time
        self.max_memory = max_memory
        self.disjunct_cost = disjunct_cost
        self.max_sentence_length = max_sentence_length
        self.allow_null = allow_null
        
        self._create_parse_options()
        self.dictionary = Dictionary("en")

    def _create_parse_options(self):
        self.parse_options = ParseOptions(verbosity=self.verbosity,
                                          linkage_limit=self.linkage_limit,
                                          min_null_count=self.min_null_count,
                                          max_null_count=self.max_null_count,
                                          null_block=self.null_block,
                                          islands_ok=self.islands_ok,
                                          short_length=self.short_length,
                                          display_walls=self.display_walls,
                                          allow_null=self.allow_null,
                                          screen_width=self.screen_width,
                                          max_parse_time=self.max_parse_time,
                                          max_memory=self.max_memory,
                                          disjunct_cost=self.disjunct_cost,
                                          max_sentence_length=self.max_sentence_length)
        
        
    def parse_sent(self, txt):
        sent = clg.sentence_create(txt,self.dictionary._dict)
        n = clg.sentence_parse(sent, self.parse_options._po)
        #This is ugly, but the parser sometiems starts failing on parses.
        #if it fails, recreate parse_options one time and try again. This will correct the issue.
        #If the sentence truly cannot be parsed, then it'll fail on the second time.
        if n == 0:
            self._create_parse_options()
            n = clg.sentence_parse(sent, self.parse_options._po)
        if n > 0:
            linkages = [Linkage.create_from_sentence(i,sent, self.parse_options._po) for i in range(n)]
        else:
            linkages = None
        clg.sentence_delete(sent)
        #linkgrammar will start to poop its pants if its 'resources' are exhausted.
        #This is a little ugly to check each time but it seems to be a cheap operation.
        #If the resources are exhausted, then reset them.
        #Extra note: apparently resetting them was not enough. If you kept parsing
        #the same sentence over and over, eventually the parser were stop returning
        #parses. I found that deleting and reinstantiating the parse_options corrected
        #this issue.
        if clg.parse_options_resources_exhausted(self.parse_options._po):
            clg.parse_options_reset_resources(self.parse_options._po)
            del self.parse_options
            self._create_parse_options()
        return linkages
    
    def __del__(self):
        del self.parse_options
        del self.dictionary

    @property
    def version(self):
        return clg.linkgrammar_get_version()

class ParseOptions(object):
    def __init__(self,verbosity=0,
                       linkage_limit=100,
                       min_null_count=0,
                       max_null_count=0,
                       null_block=1,
                       islands_ok=False,
                       short_length=6,
                       all_short_connectors=False,
                       display_walls=False,
                       allow_null=True,
                       screen_width=180,
                       max_parse_time=-1,
                       max_memory=128000000,
                       disjunct_cost=10000,
                       max_sentence_length=170):
    
        
        self._po = clg.parse_options_create()
        self.verbosity = verbosity
        self.linkage_limit = linkage_limit
        self.min_null_count = min_null_count
        self.max_null_count = max_null_count
        self.null_block = null_block
        self.islands_ok = islands_ok
        self.short_length = short_length
        self.all_short_connectors = all_short_connectors
        self.display_walls = display_walls
        self.screen_width = screen_width
        self.max_parse_time = max_parse_time
        self.max_memory = max_memory
        self.disjunct_cost = disjunct_cost
        self.max_sentence_length = max_sentence_length
        self.allow_null = allow_null
    
    def __del__(self):
        if self._po is not None:
            clg.parse_options_delete(self._po)
            self._po = None

    def verbosity():
        doc = "This sets/gets the level of description printed to stderr/stdout about the parsing process."
        def fget(self):
            return clg.parse_options_get_verbosity(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("Verbosity must be set to an integer")
            if value not in [0, 1, 2]:
                raise ValueError("Verbosity levels can be 0, 1, or 2")
            clg.parse_options_set_verbosity(self._po, value)
        return locals() 
    verbosity = property(**verbosity())
    
    def linkage_limit():
        doc = "This parameter determines the maximum number of linkages that are considered in post-processing. If more than linkage_limit linkages found, then a random sample of linkage_limit is chosen for post-processing. When this happen a warning is displayed at verbosity levels bigger than 1."
        def fget(self):
            return clg.parse_options_get_linkage_limit(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("Linkage Limit must be set to an integer")
            if value < 0:
                raise ValueError("Linkage Limit must be positive")
            clg.parse_options_set_linkage_limit(self._po, value)
        return locals() 
    linkage_limit = property(**linkage_limit())
    
    def disjunct_cost():
        doc = "Determines the maximum disjunct cost used during parsing, where the cost of a disjunct is equal to the maximum cost of all of its connectors. The default is that all disjuncts, no matter what their cost, are considered."
        def fget(self):
            return clg.parse_options_get_disjunct_cost(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("Distjunct cost must be set to an integer")
            clg.parse_options_set_disjunct_cost(self._po, value)
        return locals() 
    disjunct_cost = property(**disjunct_cost())

    def min_null_count():
        doc = "The minimum number of null links that a parse might have. A call to sentence_parse will find all linkages having the minimum number of null links within the range specified by this parameter."
        def fget(self):
            return clg.parse_options_get_min_null_count(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("min_null_count must be set to an integer")
            if value < 0:
                raise ValueError("min_null_count must be positive")
            clg.parse_options_set_min_null_count(self._po, value)
        return locals()
    min_null_count = property(**min_null_count())

    def max_null_count():
        doc = "The maximum number of null links that a parse might have. A call to sentence_parse will find all linkages having the minimum number of null links within the range specified by this parameter."
        def fget(self):
            return clg.parse_options_get_max_null_count(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("max_null_count must be set to an integer")
            if value < 0:
                raise ValueError("max_null_count must be positive")
            clg.parse_options_set_max_null_count(self._po, value)
        return locals()
    max_null_count = property(**max_null_count())
            
    def null_block():
        doc = "This allows null links to be counted in \"bunches.\" For example, if null_block is 4, then a linkage with 1,2,3 or 4 null links has a null cost of 1, a linkage with 5,6,7 or 8 null links has a null cost of 2, etc. (This is only in effect if islands are not allowed; see below.)"
        def fget(self):
            return clg.parse_options_get_null_block(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("null_block must be set to an integer")
            if value < 0:
                raise ValueError("null_block must be positive")
            clg.parse_options_set_null_block(self._po, value)
        return locals()
    null_block = property(**null_block())
            
    def short_length():
        doc = "The short_length parameter determines how long the links are allowed to be. The intended use of this is to speed up parsing by not considering very long links for most connectors, since they are very rarely used in a correct parse. An entry for UNLIMITED-CONNECTORS in the dictionary will specify which connectors are exempt from the length limit."
        def fget(self):
            return clg.parse_options_get_short_length(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("short_length must be set to an integer")
            if value < 0:
                raise ValueError("short_length must be positive")
            clg.parse_options_set_short_length(self._po, value)
        return locals()
    short_length = property(**short_length())
            
    def max_sentence_length():
        doc = ""
        def fget(self):
            return clg.parse_options_get_max_sentence_length(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("max_sentence_length must be set to an integer")
            if value < 0:
                raise ValueError("max_sentence_length must be positive")
            clg.parse_options_set_max_sentence_length(self._po, value)
        return locals()
    max_sentence_length = property(**max_sentence_length())
            
    def islands_ok():
        doc = """
        This option determines whether or not "islands" of links are allowed. For example, the following linkage has an island:
            +------Wd-----+                                           
            |     +--Dsu--+---Ss--+-Paf-+      +--Dsu--+---Ss--+--Pa-+
            |     |       |       |     |      |       |       |     |
          ///// this sentence.n is.v false.a this sentence.n is.v true.a 
        """
        def fget(self):
            return clg.parse_options_get_islands_ok(self._po) == 1
        def fset(self, value):
            if not isinstance(value, bool):
                raise TypeError("islands_ok must be set to an integer")
            clg.parse_options_set_islands_ok(self._po, 1 if value else 0)
        return locals()
    islands_ok = property(**islands_ok())
            
    def max_memory():
        doc = "Determines the maximum memory allowed during parsing. This is used just as max_parse_time is, so that the parsing process is terminated as quickly as possible after the total memory (including that allocated to all dictionaries, etc.) exceeds the maximum allowed."
        def fget(self):
            return clg.parse_options_get_max_memory(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("max_memory must be set to an integer")
            if value < 0:
                raise ValueError("max_memory must be positive")
            clg.parse_options_set_max_memory(self._po, value)
        return locals()
    max_memory = property(**max_memory())
            
    def max_parse_time():
        doc = "Determines the approximate maximum time (in seconds) that parsing is allowed to take. The way it works is that after this time has expired, the parsing process is artificially forced to complete quickly by pretending that no further solutions (entries in the hash table) can be constructed. The actual parsing time might be slightly longer."
        def fget(self):
            return clg.parse_options_get_max_parse_time(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("max_parse_time must be set to an integer")
            clg.parse_options_set_max_parse_time(self._po, value)
        return locals()
    max_parse_time = property(**max_parse_time())
    
    def screen_width():
        doc = "The width of the screen (in characters) for displaying linkages."
        def fget(self):
            return clg.parse_options_get_screen_width(self._po)
        def fset(self, value):
            if not isinstance(value, int):
                raise TypeError("screen_width must be set to an integer")
            if value < 0:
                raise ValueError("screen_width must be positive")
            clg.parse_options_set_screen_width(self._po, value)
        return locals()
    screen_width = property(**screen_width())
    
    def allow_null():
        doc = "Whether or not to allow linkages to have null links."
        def fget(self):
            return clg.parse_options_get_allow_null(self._po) == 1
        def fset(self, value):
            if not isinstance(value, bool):
                raise TypeError("allow_null must be set to an integer")
            clg.parse_options_set_allow_null(self._po, 1 if value else 0)
        return locals()
    allow_null = property(**allow_null())

    def display_walls():
        doc = "Whether or not to show the wall word(s) when a linkage diagram is printed."
        def fget(self):
            return clg.parse_options_get_display_walls(self._po) == 1
        def fset(self, value):
            if not isinstance(value, bool):
                raise TypeError("display_walls must be set to an integer")
            clg.parse_options_set_display_walls(self._po, 1 if value else 0)
        return locals()
    display_walls = property(**display_walls())
            
    def all_short_connectors():
        doc = "If true, then all connectors have length restrictions imposed on them -- they can be no farther than short_length apart. This is used when parsing in \"panic\" mode, for example."
        def fget(self):
            return clg.parse_options_get_all_short_connectors(self._po) == 1
        def fset(self, value):
            if not isinstance(value, bool):
                raise TypeError("all_short_connectors must be set to an integer")
            clg.parse_options_set_all_short_connectors(self._po, 1 if value else 0)
        return locals()
    all_short_connectors = property(**all_short_connectors())
            
            

class Dictionary(object):
    def __init__(self,lang):
        self._dict = clg.dictionary_create_lang(lang)
    
    def __del__(self):
        if self._dict is not None:
            clg.dictionary_delete(self._dict)
            self._dict = None
            
class Link(object):
    def __init__(self, left_word, left_link, right_link, right_word):
        self.left_word = left_word
        self.right_word = right_word
        self.left_link = left_link
        self.right_link = right_link
        
    def __eq__(self, other):
        return self.left_word == other.left_word and self.left_link == other.left_link and \
               self.right_word == other.right_word and self.right_link == other.right_link
                               
    def __unicode__(self):
        if self.left_link == self.right_link:
            return u"%s-%s-%s" % (self.left_word, self.left_link, self.right_word)
        else:
            return u"%s-%s-%s-%s" % (self.left_word, self.left_link, self.right_link, self.right_word)
            
    def __repr__(self):
        return "Link: %s" % unicode(self)
        
    
class Linkage(object):
    @classmethod
    def create_from_sentence(cls, idx, sentence, parse_options):
        linkage_swig_obj = clg.linkage_create(idx,sentence, parse_options)
        linkage_obj = Linkage(linkage_swig_obj, null_count=clg.sentence_null_count(sentence))
        clg.linkage_delete(linkage_swig_obj)
        return linkage_obj

    def __init__(self, linkage_swig_obj, calculate_sub_linkages=True, null_count=0):
        self.num_of_sublinkages = clg.linkage_get_num_sublinkages(linkage_swig_obj)
        self.num_of_words = clg.linkage_get_num_words(linkage_swig_obj)
        self.num_of_links = clg.linkage_get_num_links(linkage_swig_obj)
        self.link_cost = clg.linkage_link_cost(linkage_swig_obj)
        
        self.words = [clg.linkage_get_word(linkage_swig_obj,i) for i in range(self.num_of_words)]
        self.links = [Link(self.words[clg.linkage_get_link_lword(linkage_swig_obj,i)],
                           clg.linkage_get_link_llabel(linkage_swig_obj,i),
                           clg.linkage_get_link_rlabel(linkage_swig_obj,i),
                           self.words[clg.linkage_get_link_rword(linkage_swig_obj,i)]) for i in range(self.num_of_links)]
        self.null_count = null_count
        self.link_distances = [clg.linkage_get_link_length(linkage_swig_obj, i) for i in range(self.num_of_links)]
        self.constituent_phrases_flat = self._get_constituent_phrases(linkage_swig_obj, flat=True)
        self.constituent_phrases_nested = self._get_constituent_phrases(linkage_swig_obj, flat=False)
        self.diagram = clg.linkage_print_diagram(linkage_swig_obj)
        self.senses = clg.linkage_print_senses(linkage_swig_obj)
        self.links_and_domains = clg.linkage_print_links_and_domains(linkage_swig_obj)
        self.postscript_snippet = clg.linkage_print_postscript(linkage_swig_obj, 0)
        self.postscript = clg.linkage_print_postscript(linkage_swig_obj, 1)

        clg.linkage_compute_union(linkage_swig_obj)
        self.sublinkages = []
        if calculate_sub_linkages:
            for i in range(0,self.num_of_sublinkages + 1,1):
                clg.linkage_set_current_sublinkage(linkage_swig_obj, i)
                self.sublinkages.append(Linkage(linkage_swig_obj, calculate_sub_linkages=False))
            self.union = self.sublinkages.pop(-1)

    def _get_constituent_phrases(self, linkage_swig_obj, flat=False):
        node = clg.linkage_constituent_tree(linkage_swig_obj)
        result = self._build_constituent_phrases(node, flat)
        clg.linkage_free_constituent_tree(node)
        return result

    def _build_constituent_phrases(self, node, flat):
        label = clg.linkage_constituent_node_get_label(node)
        if label in ['NP','VP','S','PP', 'SBAR', 'WHNP','WHPP', 'SINV', 'QP', 'WHADVP', 'PRT', 'ADJP','ADVP']:
            ret = [ConstituentNode(label)]
        else:
            ret = [label]
        child = clg.linkage_constituent_node_get_child(node)
        if child:
            children = self._build_constituent_phrases(child, flat)
            words = [c for c in children if not isinstance(c, ConstituentNode) and not isinstance(c, list) and len(c) > 0]
            child_nodes = [c for c in children if isinstance(c, ConstituentNode) or (isinstance(c,list) and len(c) > 0)]
            ret[-1].words = words
            if flat:
                ret.extend(child_nodes)
            else:
                ret.append(child_nodes)
            
        sibling = clg.linkage_constituent_node_get_next(node)
        if sibling:
            ret.extend(self._build_constituent_phrases(sibling, flat))
            
        return ret

class ConstituentNode(object):
    def __init__(self,type, words=None):
        self.type = type
        if not words:
            words = []
        self.words = words
        
    def __repr__(self):
        if self.words:
            return "<%s: %s>" % (self.type, ', '.join(self.words))
        else:
            return "<%s>" % self.type

    def __eq__(self, other):
        return self.type == other.type and self.words == other.words


#!/usr/bin/python

import os
import sys
from distutils.command.install import INSTALL_SCHEMES
from distutils.command.install_data import install_data
from distutils.core import setup

#REVIEW: Okay this is super ugly, but it'll at least allow for this package to be pippable.
from subprocess import call
call(["cmake", os.path.join(os.path.dirname(__file__),  "pylinkgrammar")])
call(["make"])

#Further horribleness.  Surely there is a way to make make put stuff where we want it, or to make
#cmake tell its makefile to do that, but for now, this works.
import shutil
for f in ["clinkgrammar.py", "_clinkgrammar.so"]:
    shutil.move(os.path.join(os.path.dirname(__file__), f), os.path.join(os.path.dirname(__file__), "pylinkgrammar", f))

cmdclasses = {'install_data': install_data}

# Tell distutils to put the data_files in platform-specific installation
# locations. See here for an explanation:
# http://groups.google.com/group/comp.lang.python/browse_thread/thread/35ec7b2fed36eaec/2105ee4d9e8042cb
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

# We need to walk down the directory we are in, and add any packages we find to a list of
# packages. This trick was stolen from Django's setup.py.
def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

# Tell distutils to put the data_files in platform-specific installation
# locations. See here for an explanation:
# http://groups.google.com/group/comp.lang.python/browse_thread/thread/35ec7b2fed36eaec/2105ee4d9e8042cb
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages = []
data_files = []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
code_dir = 'pylinkgrammar'

for dirpath, dirnames, filenames in os.walk(code_dir):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    if filenames:
        non_python_filenames = []
        for filename in filenames:
            if not (filename.endswith('.py') or filename.endswith('.pyc')):
                non_python_filenames.append(filename)
        data_files.append([dirpath,
             [os.path.join(dirpath, f) for f in non_python_filenames]])
setup(name='pylinkgrammar', version='0.2.4',
    description='Python bindings for Link Grammar system',
    long_description=open('README.txt').read(),
    author='MetaMetrics, Inc.',
    packages=packages,
    data_files=data_files,
    cmdclass = cmdclasses)
